/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.isobus.type;

import java.util.List;
import nl.cloudfarming.client.isobus.model.*;
import org.openide.nodes.ChildFactory;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.util.NbBundle;
import org.openide.util.NbBundle.Messages;

/**
 *
 * @author Gerben Feenstra
 */
@Messages({
    "isobus.project.taskdata.aggregatenode.name.deviceprocessdata=DeviceProcessData",
    
    
    
    "isobus.project.taskdata.node.name.deviceelement=DeviceElement ({0})",
    "isobus.project.taskdata.node.name.deviceprocessdata={0} - {1}",
    "isobus.project.taskdata.node.name.devicevaluepresentation=DeviceValuePresentation ({0} - {1})",
    "isobus.project.taskdata.node.name.deviceproperty=DeviceProperty ({0} - {1})",
    
})
public class DeviceChildFactory extends ChildFactory<Node> {

    private static final String DEVICEELEMENT_NODE_ICON = "";
    private static final String DEVICEPROCESSDATA_NODE_ICON = "";
    private static final String DEVICEVALUEPRESENTATION_NODE_ICON = "";
    private static final String DEVICEPROPERTY_NODE_ICON = "";
    private final Device device;

    public DeviceChildFactory(Device device) {
        this.device = device;
    }

    @Override
    protected boolean createKeys(List<Node> toPopulate) {

        // Device Elements (DET)
        for (DeviceElement deviceElement : device.getDET()) {
            Children children = Children.create(new DeviceElementChildFactory(deviceElement), true);
            String displayName = NbBundle.getMessage(DeviceChildFactory.class, "isobus.project.taskdata.node.name.deviceelement", deviceElement.getId(), deviceElement.getDesignator());
            toPopulate.add(new IsobusCommonNode(children, deviceElement, DEVICEELEMENT_NODE_ICON, displayName));
        }

        // DeviceProcessData (DPD)
        Children children = Children.create(new ProcessDataAggregateNodeChildFactory(device.getDPD()), true);
        toPopulate.add(new IsobusAggregateNode(children, "", NbBundle.getMessage(DeviceChildFactory.class, "isobus.project.taskdata.aggregatenode.name.deviceprocessdata")));

        // DeviceValuePresentation (DVP)
        for (DeviceValuePresentation deviceValuePresentation : device.getDVP()) {
            String displayName = NbBundle.getMessage(DeviceChildFactory.class, "isobus.project.taskdata.node.name.devicevaluepresentation", deviceValuePresentation.getId(), deviceValuePresentation.getUnitDesignator());
            toPopulate.add(new IsobusCommonNode(Children.LEAF, deviceValuePresentation, DEVICEVALUEPRESENTATION_NODE_ICON, displayName));
        }

        // Deviceproperty (DPT)
        for (DeviceProperty deviceProperty : device.getDPT()) {
            String displayName = NbBundle.getMessage(DeviceChildFactory.class, "isobus.project.taskdata.node.name.deviceproperty", deviceProperty.getId(), deviceProperty.getDesignator());
            toPopulate.add(new IsobusCommonNode(Children.LEAF, deviceProperty, DEVICEPROPERTY_NODE_ICON, displayName));
        }
        return true;
    }

    @Override
    protected Node createNodeForKey(Node key) {
        return key;
    }



    private static class ProcessDataAggregateNodeChildFactory extends ChildFactory<Node> {

        private List<DeviceProcessData> deviceProcessDataList;

        public ProcessDataAggregateNodeChildFactory(List<DeviceProcessData> deviceProcessDataList) {
            this.deviceProcessDataList = deviceProcessDataList;
        }

        @Override
        protected boolean createKeys(List<Node> toPopulate) {
            for (DeviceProcessData deviceProcessData : deviceProcessDataList) {
                String displayName = NbBundle.getMessage(DeviceChildFactory.class, "isobus.project.taskdata.node.name.deviceprocessdata", deviceProcessData.getId(), deviceProcessData.getDesignator());
                toPopulate.add(new IsobusCommonNode(Children.LEAF, deviceProcessData, DEVICEPROCESSDATA_NODE_ICON, displayName));
            }
            return true;
        }

        @Override
        protected Node createNodeForKey(Node key) {
            return key;
        }
    }
    
    
    
    
}
