/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package nl.cloudfarming.client.isobus.type;

import com.vividsolutions.jts.geom.Envelope;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.MultiPolygon;
import nl.cloudfarming.client.geoviewer.Geometrical;
import nl.cloudfarming.client.isobus.model.*;
import nl.cloudfarming.client.logging.AppLogFactory;
import nl.cloudfarming.client.logging.AppLogger;
import nl.cloudfarming.client.util.GeometryUtil;
import org.openide.util.NbBundle;
import org.openide.util.NbBundle.Messages;

/**
 * Geographical representation of an Isobus partfield.
 * @author Gerben
 */
@Messages("isobus.project.taskdata.node.name.partfield_geographical={0} - {1}")
public class PartFieldGeographical implements Geometrical<Geometry> {

    private IsobusPartField isobusPartField;
    private final AppLogger LOG = AppLogFactory.getLogger(PartFieldGeographical.class);
    private Geometry geometry;

    public PartFieldGeographical(IsobusPartField isobusPartField) {
        this.isobusPartField = isobusPartField;
    }

    @Override
    public com.vividsolutions.jts.geom.Point getCentroid() {
        Geometry temp = getGeometry();
        if(temp != null){
            return temp.getCentroid();
        }
        return null;
    }

    @Override
    public Geometry getRenderObject(Envelope envelope) {
        throw new UnsupportedOperationException(" getRenderObject on PartFieldGeographical Not supported yet.");
    }

    @Override
    public String getTooltipText() {
        return NbBundle.getMessage(AggregateNodeChildFactory.class, "isobus.project.taskdata.node.name.partfield_geographical", isobusPartField.getCode(), isobusPartField.getDesignator());
    }

    @Override
    public long getId() {
        throw new UnsupportedOperationException("getId on PartFieldGeographical Not supported yet.");
    }

    // TODO: Isobus partfield can also have linestrings and points besides polygons
    /**
     * Build wkt to create a vividsolutions Geometry.
     */
    @Override
    public Geometry getGeometry() {
        if (geometry == null) {

            // Polygon should have at least 1 coordinate.
            int counter = 0;
            
            if (isobusPartField != null) {
                StringBuilder wkt = new StringBuilder();
                wkt.append("MULTIPOLYGON(");

                for (IsobusPolygon polygon : isobusPartField.getPLN()) {
                    wkt.append("(");

                    for (IsobusLineString line : polygon.getLineString()) {
                        String firstPoint = "";
                        // Polygon exterior
                        if (line.getLineStringType() == LineStringTypeEnum.POLYGON_EXTERIOR) {
                            wkt.append("(");
                            for (IsobusPoint point : line.getPoints()) {
                                counter++;
                                StringBuilder pointStr = new StringBuilder();

                                pointStr.append(point.getPositionEast());
                                pointStr.append(" ");
                                pointStr.append(point.getPositionNorth());

                                if (firstPoint.equals("")) {
                                    firstPoint = pointStr.toString();
                                }

                                wkt.append(pointStr);
                                wkt.append(",");
                            }
                            
                            // Add first coordinate again to close the polygon
                            wkt.append(firstPoint);
                            wkt.append("),");
                        }

                    }
                    // Remove last char (,)
                    wkt.deleteCharAt(wkt.length() - 1);
                    wkt.append("),");
                }

                // Remove last char (,)
                wkt.deleteCharAt(wkt.length() - 1);
                wkt.append(")");

                // Polygon should have at least 1 coordinate.
                if(counter > 0){
                    LOG.info("Extracted WKT: " + wkt.toString());
                    geometry = (MultiPolygon) GeometryUtil.wktToGeometry(wkt.toString());
                    return geometry;
                }
                return null;
            }
        }
        return geometry;
    }
}
